package com.bbc.denadrive

import android.app.PendingIntent
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.core.net.toUri
import androidx.navigation.compose.rememberNavController
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKey
import com.bbc.denadrive.apihandlers.makeDriveApiCall
import com.bbc.denadrive.home.ScaffoldWithSidebar
import com.bbc.denadrive.oauth.retrieveAuthServConf
import com.bbc.denadrive.ui.theme.DeNaDriveTheme
import net.openid.appauth.AuthState
import net.openid.appauth.AuthorizationException
import net.openid.appauth.AuthorizationRequest
import net.openid.appauth.AuthorizationResponse
import net.openid.appauth.AuthorizationService
import net.openid.appauth.ResponseTypeValues

class MainActivity : ComponentActivity() {

    private lateinit var masterKey: MasterKey
    private lateinit var preferences: SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        masterKey = MasterKey.Builder(this)
            .setKeyScheme(MasterKey.KeyScheme.AES256_GCM)
            .build()

        preferences = EncryptedSharedPreferences.create(
            this,
            "authState",
            masterKey,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val authState = preferences.getString("authState", "bah")
            ?.let { AuthState.jsonDeserialize(it) }

        val authService = AuthorizationService(this)

//        authState?.let {
//            it.performActionWithFreshTokens(authService) { accessToken, idToken, ex ->
//                if (ex != null) {
//                    Log.e("TELLINGYA", "${ex.error}")
//                    return@performActionWithFreshTokens
//                }
//                Log.e("TOLD", "okay I guess")
//            }
//        }

        authState?.let {
            val myUrl = "https://www.googleapis.com/drive/v3/files?trashed=false&orderBy=folder"
            makeDriveApiCall(this, it, myUrl, "root") {
                response, exception ->
                if (exception != null) {
                    Log.e("ECCOLO", "${exception.cause}")
                    Log.e("ECCOLO", "${exception.message}")
                }
                else {
                    Log.e("UOOOO", "$response")
                }
            }
            with(preferences.edit()) {
                this.remove("authState")
                this.putString("authState", it.jsonSerializeString())
                this.apply()
            }
        }

        setContent {
            val navController = rememberNavController()
            ScaffoldWithSidebar(navController) {
                Log.e("LETTINGYOUKNOW", "authstate: ${authState?.jsonSerializeString()}")
            }
        }
    }


    private fun handleAuthStuff() {
        val serviceConfig = retrieveAuthServConf()
//        val authState = AuthState(serviceConfig)
        val authRequestBuilder = AuthorizationRequest.Builder(
            serviceConfig,
            "722393551256-pum20gbvb1es723kt3ek9lmtrdimr0os.apps.googleusercontent.com",
            ResponseTypeValues.CODE,
            "com.bbc.denadrive:/oauth2redirect".toUri()
        )
        val authRequest = authRequestBuilder
            .setScope("https://www.googleapis.com/auth/drive")
            .build()

        val authService = AuthorizationService(this)
        authService.performAuthorizationRequest(
            authRequest,
            PendingIntent.getActivity(this, 0, Intent(this,
                TestActivity::class.java), PendingIntent.FLAG_MUTABLE),
            PendingIntent.getActivity(this, 0, Intent(this,
                TestBah::class.java), PendingIntent.FLAG_MUTABLE)
        )
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        val resp = AuthorizationResponse.fromIntent(intent)
        val ex = AuthorizationException.fromIntent(intent)
        if (resp != null) {
            Toast.makeText(this, "YES", Toast.LENGTH_SHORT).show()
        }
        else {
            if (ex != null) {
                Toast.makeText(this, "NOPE ${ex.error}", Toast.LENGTH_SHORT).show()
            }
        }

        setContent {
//            val navController = rememberNavController()
//            ScaffoldWithSidebar(navController) {  }
            Greeting("bah")
        }
    }
}

@Composable
fun Greeting(name: String, modifier: Modifier = Modifier) {
    Text(
        text = "Hello $name!",
        modifier = modifier
    )
}

@Preview(showBackground = true)
@Composable
fun GreetingPreview() {
    DeNaDriveTheme {
        Greeting("Android")
    }
}