package com.bbc.denadrive.apihandlers

import android.content.Context
import net.openid.appauth.AuthState
import net.openid.appauth.AuthorizationService
import okhttp3.Call
import okhttp3.Callback
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.Response
import okhttp3.HttpUrl.Companion.toHttpUrl
import java.io.IOException

fun makeDriveApiCall(context: Context, authState: AuthState, url: String, folderId: String,
                     callback: (String?, Exception?) -> Unit) {
    val authService = AuthorizationService(context)
    // _ = idToken
    authState.performActionWithFreshTokens(authService) { accessToken, _, ex ->
        authService.dispose()
        if (ex != null) {
            callback(null, ex)
            return@performActionWithFreshTokens
        }

        val urlBuilder = url.toHttpUrl().newBuilder()
        urlBuilder.addQueryParameter("q", "'$folderId' in parents")
        val updatedUrl = urlBuilder.build()

        val request = Request.Builder()
            .url(updatedUrl)
            .header("Authorization", "Bearer $accessToken")
            .build()

        OkHttpClient().newCall(request).enqueue(object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                callback(null, e)
            }

            override fun onResponse(call: Call, response: Response) {
                val responseBody = response.body?.string()
                callback(responseBody, null)
            }
        })
    }
}
