package com.bbc.denadrive.home

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKey
import com.bbc.denadrive.HomeNotAuthActivity
import com.bbc.denadrive.MainActivity
import com.bbc.denadrive.home.ui.theme.DeNaDriveTheme
import net.openid.appauth.AuthState

class NeverLoggedInLogic : ComponentActivity() {

    private lateinit var preferences: SharedPreferences
    private lateinit var masterKey: MasterKey

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        masterKey = MasterKey.Builder(this)
            .setKeyScheme(MasterKey.KeyScheme.AES256_GCM)
            .build()

        preferences = EncryptedSharedPreferences.create(
            this,
            "authState",
            masterKey,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        val authStateString = preferences.getString("authState", "unset")

        val intent: Intent
        if (authStateString == "unset") {
            intent = Intent(this, HomeNotAuthActivity::class.java)
        } else {
            val authState = authStateString?.let { AuthState.jsonDeserialize(it) }
            intent = if (authState?.isAuthorized == true) {
                Intent(this, MainActivity::class.java)
            }
            else {
                Toast.makeText(this, "Session expired, please log in again",
                    Toast.LENGTH_SHORT).show()
                Intent(this, HomeNotAuthActivity::class.java)
            }
        }
        startActivity(intent)
//        Toast.makeText(this, "should not make it here", Toast.LENGTH_SHORT).show()

        enableEdgeToEdge()
        setContent {
            DeNaDriveTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    Greeting5(
                        name = "Android",
                        modifier = Modifier.padding(innerPadding)
                    )
                }
            }
        }
    }
}

@Composable
fun Greeting5(name: String, modifier: Modifier = Modifier) {
    Text(
        text = "Hello $name!",
        modifier = modifier
    )
}

@Preview(showBackground = true)
@Composable
fun GreetingPreview5() {
    DeNaDriveTheme {
        Greeting5("Android")
    }
}